* macros_match_cases_with_controls.sps.

DEFINE !pathd()'c:\temp\'!ENDDEFINE.

* Generate sample data for illustration purposes.
SET SEED=01251701.
NEW FILE.
INPUT PROGRAM.
LOOP id=1 TO 300./* generate 300 cases*/
+ COMPUTE sex=1 + (UNIFORM(1)>.5).
+ COMPUTE age=TRUNC(50+UNIFORM(20)). 
+ COMPUTE event=UNIFORM(1)>.96./* 1=case, 0=potential control */
+ COMPUTE fup  =UNIFORM(10).
+ DO IF event=1.
+   COMPUTE evtday  =RND(UNIFORM(28)).
+   COMPUTE evtmonth=RND(.5 + UNIFORM(12)).
+   COMPUTE evtyear =RND(1993 + UNIFORM(10)).
+   COMPUTE fup=(CTIME.DAYS(DATE.DMY(evtday,evtmonth,evtyear) - 
  DATE.DMY(1,1,1993)))/365.25.
+ END IF.
+ END CASE.
END LOOP.
END FILE.
FORMATS id TO event evtday TO evtyear (F5) fup(F4.2).
END INPUT PROGRAM.
FREQ VAR=event.
SAVE OUTFILE=!pathd + "population.sav".

* Define macro which finds control subjects for a given id.
DEFINE !do_one (id=!TOKENS(1) /age=!TOKENS(1) /delta=!TOKENS(1) 
  /fup=!TOKENS(1)  /sex=!TOKENS(1)  /first=!TOKENS(1))
GET FILE=!pathd + "remaining pop.sav".
SELECT IF (event=0) AND
  RANGE(age,!age - !delta, !age + !delta) AND 
  (fup>!fup) AND
  (sex=!sex).
COMPUTE draw=UNIFORM(1).
SORT CASES BY draw.
/* keep only 3 controls (Change number as required) */
N OF CASES 3.
COMPUTE matchid=!id.
ADD FILES FILE=* /DROP=evtmonth evtyear evtday.
SORT CASES BY id.
SAVE OUTFILE=!pathd + "control.sav" /DROP=draw.
* Add controls to list of those found.
!IF (!first !NE 1) !THEN 
ADD FILES FILE=*
  /FILE=!pathd + "all control.sav".
!IFEND
SAVE OUTFILE=!pathd + "all control.sav".
* Remove those control from main file.
MATCH FILES FILE=!pathd + "control.sav"
  /IN=control
  /FILE=!pathd + "remaining pop.sav"
  /BY=id.
SELECT IF control=0.
SAVE OUTFILE=!pathd + "remaining pop.sav" /DROP=control.
!ENDDEFINE.

*Change next number or comment the line to obtain different results.
SET SEED=987654321 PRINTBACK=ON.
GET FILE=!pathd + "population.sav".
SORT CASES BY id.
SAVE OUTFILE=!pathd + "remaining pop.sav".
SHOW SEED.

* Write syntax to call macro for each case where event=1.
SELECT IF event=1.
COMPUTE first=($CASENUM=1).
COMPUTE delta=1 /* Change value of delta as required */.
TEMPORARY.
FORMATS first(F1) delta(F3) fup(COMMA8.3).
WRITE OUTFILE=!pathd + "call macro.sps"
  /"!do_one id="id" age="age" delta="delta" fup="fup
  " sex="sex" first="first".".
EXECUTE.
SET MPRINT=ON.
* Call macro for each case where event=1.
INSERT FILE= !pathd + "call macro.sps".
* We need matchid to be able to match cases and control.
GET FILE=!pathd + "remaining pop.sav".
COMPUTE matchid=id.
SAVE OUTFILE=!pathd + "remaining pop.sav".
* Add evtday evtmonth and evtyear to control subjects.
GET FILE=!pathd + "all control.sav" /DROP=draw.
SORT CASES BY matchid.
FORMATS matchid(F5).
MATCH FILES FILE=*
  /TABLE=!pathd + "remaining pop.sav"
  /BY=matchid.
* Add control cases to remaining population.
ADD FILES FILE=*
  /FILE=!pathd + "remaining pop.sav".
SORT CASES BY matchid(A) event(D) id(A).
* Show # of control subjects found per id.
TEMPORARY.
SELECT IF NOT MISSING(evtyear).
SUMMARIZE
  /TABLES=fup  BY matchid
  /FORMAT=NOLIST TOTAL
  /TITLE='matchid with N=1 have no control subjects'
  /MISSING=VARIABLE
  /CELLS=COUNT MIN.
